﻿using ASPItalia.Apps4Windows.WinRT.Common;

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using ASPItalia.Apps4Windows.WinRT.Pages;
using Windows.UI.ApplicationSettings;
using Callisto.Controls;
using ASPItalia.Apps4Windows.WinRT.Services;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using System.Threading.Tasks;
using Windows.UI;
using Windows.System;
using ASPItalia.Apps4Windows.WinRT.Resources;

namespace ASPItalia.Apps4Windows.WinRT
{
    /// <summary>
    /// Provides application-specific behavior to supplement the default Application class.
    /// </summary>
    sealed partial class App : Application
    {
        private Frame rootFrame;
        /// <summary>
        /// Initializes the singleton Application object.  This is the first line of authored code
        /// executed, and as such is the logical equivalent of main() or WinMain().
        /// </summary>
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;
            this.UnhandledException += App_UnhandledException;
        }

        /// <summary>
        /// Invoked when the application is launched normally by the end user.  Other entry points
        /// will be used when the application is launched to open a specific file, to display
        /// search results, and so forth.
        /// </summary>
        /// <param name="args">Details about the launch request and process.</param>
        protected override async void OnLaunched(LaunchActivatedEventArgs args)
        {
            await this.PerformLaunch(args.Arguments, args.PreviousExecutionState);
        }

        public async Task PerformLaunch(string arguments, ApplicationExecutionState previousState)
        {
            // Do not repeat app initialization when already running, just ensure that
            // the window is active
            if (previousState == ApplicationExecutionState.Running)
            {
                Window.Current.Activate();
                // Se arrivo da pin, navigo alla pagina chiesta
                if (!String.IsNullOrWhiteSpace(arguments))
                {
                    NavigateByArguments(arguments);
                }
                return;
            }

            // Create a Frame to act as the navigation context and associate it with
            // a SuspensionManager key
            this.rootFrame = new Frame();
            SuspensionManager.RegisterFrame(rootFrame, "AppFrame");

            // Restore dellos stato solo se non arrivo da pin
            if (previousState == ApplicationExecutionState.Terminated && String.IsNullOrWhiteSpace(arguments))
            {
                try
                {
                    // Restore the saved session state only when appropriate
                    await SuspensionManager.RestoreAsync();
                }
                catch
                { }

            }

            if (rootFrame.Content == null)
            {
                // When the navigation stack isn't restored navigate to the first page,
                // configuring the new page by passing required information as a navigation
                // parameter
                if (!rootFrame.Navigate(typeof(MainPage)))
                {
                    throw new Exception("Failed to create initial page");
                }

                if (!String.IsNullOrWhiteSpace(arguments))
                {
                    NavigateByArguments(arguments);
                }
            }

            // Place the frame in the current Window and ensure that it is active
            Window.Current.Content = rootFrame;
            Window.Current.Activate();

            InitSettings();
        }

        private void NavigateByArguments(string arguments)
        {
            string[] pipes = arguments.Split(new char[] { '|' }, 2);
            if (pipes.Length < 1) return;

            Section section;
            ILoader loader;

            SectionNames name = (SectionNames)Enum.Parse(typeof(SectionNames), pipes[0]);
            if (pipes.Length > 1)
            {
				// ENG: Navigate to the section given the ID
                // ITA: Navigo alla sezione e id richiesti del dettaglio
                Repository.GetSectionAndLoader(name, out section, out loader);
                SectionItem item = new SectionItem(section, new FeedItem { Id = pipes[1] }, 0);
                this.rootFrame.NavigateToDetails(item);
            }
            else
            {
                Repository.GetSectionAndLoader(name, out section, out loader);

				// ENG: navigate to the section
                // ITA: Navigo solo nella sezione
                this.rootFrame.NavigateToSection(section);
            }
        }

        /// <summary>
        /// Invoked when application execution is being suspended.  Application state is saved
        /// without knowing whether the application will be terminated or resumed with the contents
        /// of memory still intact.
        /// </summary>
        /// <param name="sender">The source of the suspend request.</param>
        /// <param name="e">Details about the suspend request.</param>
        private async void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            await SuspensionManager.SaveAsync();
            deferral.Complete();
        }

        private void App_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
			// ENG: to prevent crash, exceptions are blocked
            // ITA: Inibisco qualsiasi errore, in modo da evitare il crash dell'applicazione
            e.Handled = true;
        }

        private void InitSettings()
        {
            SettingsPane.GetForCurrentView().CommandsRequested += App_CommandsRequested;
        }

        private void App_CommandsRequested(SettingsPane sender, SettingsPaneCommandsRequestedEventArgs args)
        {
			args.Request.ApplicationCommands.Add(new SettingsCommand("", StringDictionary.Options, OnOptions<OptionsPane>));
			args.Request.ApplicationCommands.Add(new SettingsCommand("", StringDictionary.Privacy, OnPrivacy));
			args.Request.ApplicationCommands.Add(new SettingsCommand("", StringDictionary.Info, OnOptions<AboutPane>));
        }

        private void OnPrivacy(Windows.UI.Popups.IUICommand command)
        {
            Launcher.LaunchUriAsync(Repository.PrivacyPolicyUri);
        }

        private void OnOptions<T>(Windows.UI.Popups.IUICommand command)
            where T : UserControl, new()
        {
            SettingsFlyout flyout = new SettingsFlyout();
            flyout.FlyoutWidth = SettingsFlyout.SettingsFlyoutWidth.Narrow;
            flyout.Content = new T();
            flyout.HeaderText = command.Label;

			// ENG: let's set the style
            // ITA: applichiamo lo stile
            var resources = App.Current.Resources.MergedDictionaries.First(x => x.Source == new Uri("ms-resource:/Files/Common/StandardStyles.xaml")).MergedDictionaries.First(x => x.Source == new Uri("ms-resource:/Files/Assets/CustomStyles.xaml"));
            flyout.Background = (SolidColorBrush)resources["ListViewItemOverlayBackgroundBrush"];
            flyout.Foreground = (SolidColorBrush)resources["ListViewItemOverlayForegroundBrush"];
            flyout.HeaderBrush = (SolidColorBrush)flyout.Background;
            flyout.ContentBackgroundBrush = (SolidColorBrush)flyout.Background;
            flyout.ContentForegroundBrush = (SolidColorBrush)flyout.Foreground;

			// ENG: we'll handle the event to set the LayerManager's state
            // ITA: intercettiamo l'evento per ripristinare lo stato del LayerManager
            flyout.Closed += (s, e) => LayerManager.OnLayerChanged(true);
            flyout.IsOpen = true;

			// ENG: let's notify the latout manager that is app is in background
			// and other panels are in foreground
            // ITA: Informo il gestore del layout che l'app non è più in primo piano
            // ma che ci sono altri pannelli in primo piano
            LayerManager.OnLayerChanged(false);
        }

        /// <summary>
        /// Invoked when the application is activated to display search results.
        /// </summary>
        /// <param name="args">Details about the activation request.</param>
        protected override void OnSearchActivated(Windows.ApplicationModel.Activation.SearchActivatedEventArgs args)
        {
            ASPItalia.Apps4Windows.WinRT.Pages.SearchResultPage.Activate(args.QueryText, args.PreviousExecutionState);
        }

    }
}