﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Model;
using Windows.Foundation;
using Windows.UI.Popups;
using Windows.UI.StartScreen;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using ASPItalia.Apps4Windows.WinRT.Resources;

namespace ASPItalia.Apps4Windows.WinRT.Common
{
	/// <summary>
	/// ENG: this class handles pinning requests
	/// ITA: Attraverso quest classe gestiamo il pinninig
	/// </summary>
	public static class PinHelper
	{
		/// <summary>
		/// ENG: Return true if the element is pinned
		/// ITA: Indica se l'elemento è in pinning
		/// </summary>
		/// <param name="content"></param>
		/// <returns></returns>
		public static async Task<bool> IsPinned(IPinnableContent content)
		{
			var results = await SecondaryTile.FindAllAsync();
			return results.Count(x => x.TileId == GetPinId(content)) > 0;
		}

		internal static string GetPinId(IPinnableContent content)
		{
			return content.UniqueID.GetHashCode().ToString();
		}

		/// <summary>
		/// ENG: Remove the elements from the start screen.
		/// ITA: Rimuove l'elemento dalla start screen.
		/// </summary>
		/// <param name="content"></param>
		/// <returns></returns>
		public async static Task RemovePin(IPinnableContent content)
		{
			var results = await SecondaryTile.FindAllAsync();
			var tile = results.FirstOrDefault(x => x.TileId == GetPinId(content));
			if (tile != null)
			{
				await tile.RequestDeleteAsync();
			}
		}

		/// <summary>
		/// ENG: Add the element to the start screen.
		/// ITA: Aggiunge l'elemento alla start screen.
		/// </summary>
		/// <param name="content"></param>
		/// <param name="anchorElement"></param>
		/// <returns></returns>
		public async static Task<bool> Pin(IPinnableContent content, FrameworkElement anchorElement)
		{
			Debug.WriteLine(content.UniqueID);

			SecondaryTile tile = new SecondaryTile(GetPinId(content), content.Title, content.Title, content.UniqueID, TileOptions.ShowNameOnLogo | TileOptions.ShowNameOnWideLogo | TileOptions.CopyOnDeployment, new System.Uri("ms-appx:///Assets/Logo.png"), new System.Uri("ms-appx:///Assets/WideLogo.png"));
			tile.ForegroundText = ForegroundText.Dark;

			// ENG: title creation
			// ITA: creazione tile
			bool success = await tile.RequestCreateForSelectionAsync(GetElementRect(anchorElement), Placement.Left);
			if (success)
			{
				// ENG: update the tile
				// ITA: aggiorno la tile appena creata
				TileHelper.UpdateSecondaryTile(content, tile.WideLogo, tile.Logo);

				await new MessageDialog(StringDictionary.PinOK).ShowAsync();
				return true;
			}
			else
			{
				// error
				await new MessageDialog(StringDictionary.PinKO).ShowAsync();
				return false;
			}
		}

		/// <summary>
		/// ENG: get the coordinates to draw the pinning window
		/// ITA: restituisce un Rect che possiamo usare per posizionare la finestra di pinning 
		/// </summary>
		/// <param name="element"></param>
		/// <returns></returns>
		private static Rect GetElementRect(FrameworkElement element)
		{
			GeneralTransform buttonTransform = element.TransformToVisual(null);
			Point point = buttonTransform.TransformPoint(new Point());
			return new Rect(point, new Size(0, element.ActualHeight));
		}
	}
}
