﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Common;

namespace ASPItalia.Apps4Windows.WinRT.Model
{
	public interface IPositionItem
	{
		int Index { get; set; }
	}
	public class SectionItem : IFreeResizable, IPinnableContent, IPositionItem
	{
		private FeedItem item;

		public SectionItem(Section section, FeedItem item, int index)
		{
			this.Section = section;
			this.item = item;

			// ENG: default values
			// ITA: valori predefiniti
			this.FillHorizontal = true;
			this.FillVertical = true;
			this.RowSpan = 1;
			this.ColumnSpan = 1;
			Index = index;
		}

		public int Index
		{
			get;
			set;
		}
		public FeedItem InnerItem { get { return item; } }

		public string Title { get { return item.Title; } }

		public string Subtitle { get { return item.Subtitle; } }

		public Uri Image { get { return item.Image; } }

        public Uri Video { get { return item.Video; } }

		public Section Section { get; private set; }

		public string Content { get { return item.Content; } }

		public string Description { get { return item.PlainTextContent; } }

		public Uri Uri { get { return item.Uri; } }

		public DateTime PubDate { get { return item.PubDate; } }

		public bool IsPhotoGallery
		{
			get
			{
				return Section.Type == SectionTypeEnum.PhotoGallery;
			}
		}

        public bool IsVideoGallery
        {
            get
            {
                return Section.Type == SectionTypeEnum.VideoGallery;
            }
        }

		public bool IsWebContent
		{
			get
			{
				return (!this.IsPhotoGallery && this.InnerItem != null &&
					this.InnerItem.ContentType == ContentType.Html) || this.IsVideoGallery;
			}
		}

		public bool IsMap
		{
			get
			{
				return this.InnerItem != null && this.InnerItem.ContentType == ContentType.Map;
			}
		}

		public Uri MapUri
		{
			get
			{
				// ENG: to prevent problems while displaying in browser
				// ITA: evita problemi di visualizzazione del browser
				if (IsMap)
					return item.Uri;
				return null;
			}
		}


		public bool IsTextContent
		{
			get
			{
				return !this.IsPhotoGallery && !this.IsVideoGallery && this.InnerItem != null &&
					this.InnerItem.ContentType == ContentType.PlainText;
			}
		}

		public int? RowSpan
		{
			get;
			set;
		}

		public int? ColumnSpan
		{
			get;
			set;
		}

		public bool FillHorizontal
		{
			get;
			set;
		}

		public bool FillVertical
		{
			get;
			set;
		}

		public string UniqueID
		{
			get
			{
				return this.Section.UniqueID + "|" + this.InnerItem.Id;
			}
		}
	}


}
