﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Common;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using ASPItalia.Apps4Windows.WinRT.Services;
using Windows.ApplicationModel.DataTransfer;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.ViewManagement;
using ASPItalia.Apps4Windows.WinRT.Resources;

namespace ASPItalia.Apps4Windows.WinRT.Pages
{

	public sealed partial class ItemDetailPage : ASPItalia.Apps4Windows.WinRT.Common.LayoutAwarePage
	{

		private DispatcherTimer slideShowTimer;

		public ItemDetailPage()
		{
			this.InitializeComponent();
		}

		/// <summary>
		/// Populates the page with content passed during navigation.  Any saved state is also
		/// provided when recreating a page from a prior session.
		/// </summary>
		/// <param name="navigationParameter">The parameter value passed to
		/// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
		/// </param>
		/// <param name="pageState">A dictionary of state preserved by this page during an earlier
		/// session.  This will be null the first time a page is visited.</param>
		protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
		{
			// ENG: the paramaters are deserialized from a string
			// ITA: Deserializzo i parametri passati come stringa
			object[] parameters = Utils.DeserializeValues((string)navigationParameter);
			
			// ENG: get section and its ID
			// ITA: Recupero nome sezione e l'id della risorsa
			this.sectionName = (SectionNames)parameters[0];
			this.selectedId = (string)parameters[1];
			
			// ENG: get the element from the state
			// ITA: Recupero dallo stato l'elemento su cui era posizionato
			if (pageState != null && pageState.ContainsKey("SelectedItem"))
			{
				selectedId = (string)pageState["SelectedItem"];
			}

			this.RefreshDetail();
		}

		private async void RefreshDetail()
		{
			try
			{
				try
				{
					// ENG: show the progress ring
					// ITA: Mostro il progress di caricamento
					this.ShowProgress = true;

					// ENG: loads section and its items
					// ITA: Carico la sezione con i relaivi elementi
					this.Section = await SectionsHelper.GetAndPrepareSessionAsync(sectionName);

					// ENG: get the selected item using its ID
					// ITA: cerco in base all'id l'elemento su cui posizionarmi
					SectionItem item = this.Section.Items.FirstOrDefault(i => i.InnerItem.Id == selectedId);
					if (item == null)
					{
						item = await LoadFromStorageAsync(selectedId);
						if (item != null)
							this.Section.Items.Insert(0, item);
					}

					// ENG: workaround for FlipView's bug
					// ITA: Workaround per bug di selezione del FlipView
					await Task.Yield();

					this.flipView.SelectedItem = item;
					if (item == null)
					{
						this.ErrorMessage = StringDictionary.ContentNotAvailable;
						this.ShowError = true;
					}

					// ENG: the AppBar is updated
					// ITA: aggiorniamo AppBar
					UpdateAppBar();
				}
				finally
				{
					// ENG: the progress is collapsed anyway
					// ITA: Qualunque cosa succeda nascondo il progress di caricamento
					this.ShowProgress = false;

					if (BottomAppBar != null && BottomAppBar.IsOpen)
						LayerManager.OnLayerChanged(false);
				}
			}
			catch (Exception e)
			{
				HandleError(e);
			}
		}

		private async Task<SectionItem> LoadFromStorageAsync(string feedItemId)
		{
			// ENG: check for the file in the local storage
			// ITA: guardo se è presente il file nello storage locale
			string fileName = String.Format("{0}.{1}", this.Section.Name, feedItemId.ToFileName());
			var file = await Persister.GetFile(fileName, false);

			if (file != null)
			{
				try
				{
					// ENG: read the stream
					// ITA: Apro lo stream in lettura
					IInputStream inStream = await file.OpenSequentialReadAsync();
					using (Stream stream = inStream.AsStreamForRead())
					{
						// ENG: deserializing from XML
						// ITA: deserializzo dall'XML
						DataContractSerializer serializer = new DataContractSerializer(typeof(FeedItem));
						FeedItem item = (FeedItem)serializer.ReadObject(stream);

						return new SectionItem(this.Section, item, 0);
					}
				}
				catch (Exception)
				{
					return null;
				}
			}

			return null;
		}

		private async Task SaveToStorage(FeedItem item)
		{
			string fileName = String.Format("{0}.{1}", this.Section.Name, item.Id.ToFileName());
			var file = await Persister.GetFile(fileName, true);

			using (IRandomAccessStream raStream = await file.OpenAsync(FileAccessMode.ReadWrite))
			{
				using (IOutputStream outStream = raStream.GetOutputStreamAt(0))
				{
					// ENG: serializing in XML
					// ITA: serializzo in XML
					using (Stream stream = outStream.AsStreamForWrite())
					{
						DataContractSerializer serializer = new DataContractSerializer(typeof(FeedItem));
						serializer.WriteObject(stream, item);
					}
				}
			}
		}

		public override void Refresh()
		{
			base.Refresh();

			this.RefreshDetail();
		}

		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			base.OnNavigatedTo(e);
			SetupShareContract();

			// ENG: get the AppBar events
			// ITA: intercettiamo l'apertura e chiusura dell'appbar
			if (this.BottomAppBar != null)
			{
				this.BottomAppBar.Opened += OnAppBarOpened;
				this.BottomAppBar.Closed += OnAppBarClosed;
			}
		}

		protected override void OnNavigatedFrom(NavigationEventArgs e)
		{
			base.OnNavigatedFrom(e);
			UnSetupShareContract();

			if (this.BottomAppBar != null)
			{
				this.BottomAppBar.Opened -= OnAppBarOpened;
				this.BottomAppBar.Closed -= OnAppBarClosed;
			}
		}


		void OnAppBarOpened(object sender, object e)
		{
			// ENG: layer is in background
			// ITA: il layer è in secondo piano
			LayerManager.OnLayerChanged(false);
		}

		void OnAppBarClosed(object sender, object e)
		{
			// ENG: layer is in foreground
			// ITA: il layer è di nuovo in primo piano
			LayerManager.OnLayerChanged(true);
		}

		private void SetupShareContract()
		{
			// ENG: share contract setup
			// ITA: impostiamo il contratto di share
			DataTransferManager.GetForCurrentView().DataRequested += new TypedEventHandler<DataTransferManager, DataRequestedEventArgs>(this.DataRequested);
		}

		private void UnSetupShareContract()
		{
			// ENG: to avoid leaks, the share contract is de-registered
			// ITA: de-registriamo il contratto di share, per evitare leak
			DataTransferManager.GetForCurrentView().DataRequested -= new TypedEventHandler<DataTransferManager, DataRequestedEventArgs>(this.DataRequested);
		}

		private void DataRequested(DataTransferManager sender, DataRequestedEventArgs e)
		{
			var item = CurrentSectionItem;
            Utils.ShareDetail(e, item);
		}

		private SectionItem CurrentSectionItem
		{
			get
			{
				return this.flipView.SelectedItem as SectionItem;
			}
		}

		Section _section;
		private SectionNames sectionName;
		private string selectedId;
		public Section Section
		{
			get
			{
				return _section;
			}
			set
			{
				base.SetProperty(ref _section, value);
			}
		}

		protected override void SaveState(Dictionary<String, Object> pageState)
		{
			var selectedItem = (SectionItem)this.flipView.SelectedItem;
			if (selectedItem != null)
				pageState["SelectedItem"] = selectedItem.InnerItem.Id;
		}

		private async void SlideShow_Click(object sender, RoutedEventArgs e)
		{
			// ENG: close the AppBar
			// ITA: Chiudo l'AppBar
			this.BottomAppBar.IsOpen = false;

			// ENG: change the visual state
			// ITA: Cambio lo stato visuale
			VisualStateManager.GoToState(this, "Active", false);

			// ENG: let's start the time to change the images
			// ITA: avvio il timer per cambiare immagine ogni tot secondi
			slideShowTimer = new DispatcherTimer();
			slideShowTimer.Tick += slideShowTimer_Tick;
			slideShowTimer.Interval = TimeSpan.FromSeconds(3);
			slideShowTimer.Start();

			UpdateAppBar();
		}

		private void slideShowTimer_Tick(object sender, object e)
		{
			slideShowTimer.Stop();

			// ENG: move the next image
			// ITA: Passo alla prossima immagine del FlipView
			if (this.flipView.SelectedIndex + 1 < this.Section.Items.Count)
				this.flipView.SelectedIndex++;
			else
				// ENG: end, stops the timer
				// ITA: fine, stop timer
				StopSlideShow_Click(sender, null);
		}

		private void Refresh_Click(object sender, RoutedEventArgs e)
		{
			this.Refresh();
		}

		private async Task RemovePin()
		{
			await PinHelper.RemovePin(CurrentSectionItem);
		}

		private async Task<bool> Pin()
		{
			if (CurrentSectionItem == null) return false;

			// ENG: save the item locally, to load it from the start screen
			// ITA: Salvo in locale l'elemento, per averlo sempre
			await SaveToStorage(CurrentSectionItem.InnerItem);

			try
			{
				this.BottomAppBar.IsSticky = true;
				return await PinHelper.Pin(CurrentSectionItem, this.PinButton);
			}
			finally
			{
				this.BottomAppBar.IsSticky = false;
			}

			return false;
		}

		private async void UpdateAppBar(bool isPinned)
		{
			bool slideActive = slideShowTimer != null;
			PinButton.Visibility = !isPinned && !slideActive ? Visibility.Visible : Visibility.Collapsed;
			UnPinButton.Visibility = isPinned && !slideActive ? Visibility.Visible : Visibility.Collapsed;
			SlideShowButton.Visibility = !slideActive && (this.Section.Type == SectionTypeEnum.PhotoGallery) ? Visibility.Visible : Visibility.Collapsed;
			RefreshButton.Visibility = !slideActive ? Visibility.Visible : Visibility.Collapsed;
			StopButton.Visibility = slideActive ? Visibility.Visible : Visibility.Collapsed;
			OpenInBrowserButton.Visibility = CurrentSectionItem != null && !CurrentSectionItem.Uri.IsLocalOrEmpty() ? Visibility.Visible : Visibility.Collapsed;
		}

		private async void PinOrUnPin(object sender, RoutedEventArgs e)
		{
			var isPinned = await PinHelper.IsPinned(CurrentSectionItem);
			if (isPinned)
			{
				await RemovePin();
				UpdateAppBar(!isPinned);
			}
			else
			{
				UpdateAppBar(await Pin());
			}
		}

		private void SlideShowImage_Opened(object sender, RoutedEventArgs e)
		{
			// ENG: image was loaded, so we can start the timer to pass to next image
			// ITA: l'immagine è stata caricata quindi posso passare all'immagine successiva
			if (slideShowTimer != null)
				slideShowTimer.Start();
		}

		private async void StopSlideShow_Click(object sender, RoutedEventArgs e)
		{
			// ENG: stops the timer
			// ITA: Fermo il timer
			if (slideShowTimer != null)
				slideShowTimer.Stop();
			slideShowTimer = null;

			// ENG: resets the slideshow visual state
			// ITA: resetto lo stato visuale dello slideshow
			VisualStateManager.GoToState(this, "Inactive", false);

			UpdateAppBar();

			// ENG: closes the bar
			// ITA: Chiudo la barra
			this.BottomAppBar.IsOpen = false;
		}

		private async void UpdateAppBar()
		{
			var isPinned = CurrentSectionItem != null ? await PinHelper.IsPinned(CurrentSectionItem) : false;
			UpdateAppBar(isPinned);
		}

		private async void OpenInBrowserButton_Click(object sender, RoutedEventArgs e)
		{
			await Launcher.LaunchUriAsync(CurrentSectionItem.Uri);
		}

		private void flipView_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			UpdateAppBar();

			OnPropertyChanged("IsImmersive");
		}
		
		public bool IsImmersive
		{
			get
			{
				var item = this.flipView.SelectedItem as SectionItem;
				var immersive = item != null && item.Section.Type == SectionTypeEnum.News && (ApplicationView.Value == ApplicationViewState.Filled || ApplicationView.Value == ApplicationViewState.FullScreenLandscape);

				return immersive;
			}
		}

		protected override void WindowSizeChanged(object sender, Windows.UI.Core.WindowSizeChangedEventArgs e)
		{
			OnPropertyChanged("IsImmersive");

			base.WindowSizeChanged(sender, e);
		}
	}
}
