using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Model;

namespace ASPItalia.Apps4Windows.WinRT.Services
{

	public static class Repository
	{

		public static readonly Uri PrivacyPolicyUri = new Uri("http://www.icubed.it/policy/");

		// ENG: Sections and loading policy are defined here
		// ITA: Definiamo le sezioni e le policy di caricamento
		private static Dictionary<SectionNames, ILoader> loaders = new Dictionary<SectionNames, ILoader>
        {
             {SectionNames.Ultimenotiziedamsdn, new RemoteFeedLoader(new Uri("http://blogs.msdn.com/italy/rss.xml"), IconTypeEnum.Info)},
             {SectionNames.Videosuwindows, new RemoteFeedLoader(new Uri("http://gdata.youtube.com/feeds/base/users/WindowsVideos/uploads?alt=rss&amp;v=2&amp;orderby=published&amp;client=ytapi-youtube-profile"), IconTypeEnum.Video)},
             {SectionNames.Informazioni, new LocalFeedLoader("Informazioni.xml", IconTypeEnum.Info)},
        };

		// ENG: sections in the navigation bar are defined here
		// ITA: definiamo le sezioni visibili nella barra di navigazione
		private static Dictionary<SectionNames, Func<Section>> sections = new Dictionary<SectionNames, Func<Section>>
        {
            { SectionNames.Informazioni, () => new Section(SectionNames.Informazioni, "Informazioni", SectionTypeEnum.Grid) },
        };

		// ENG: home page sections are defined here
		// ITA: definiamo le sezioni visibili in home page
		private static Dictionary<SectionNames, Func<Section>> homeSections = new Dictionary<SectionNames, Func<Section>>
        {
            { SectionNames.Ultimenotiziedamsdn, () => new Section(SectionNames.Ultimenotiziedamsdn, "Ultime notizie da MSDN", SectionTypeEnum.Grid, LayoutTypeEnum.Classic) },
            { SectionNames.Videosuwindows, () => new Section(SectionNames.Videosuwindows, "Video su Windows", SectionTypeEnum.VideoGallery, LayoutTypeEnum.Classic) },
        };

		private static Dictionary<SectionNames, Func<Section>> AllSections
		{
			get
			{
				return sections.Concat(homeSections).ToDictionary(s => s.Key, s => s.Value);
			}
		}

		public static void GetSectionAndLoader(SectionNames name, out Section section, out ILoader loader)
		{
			section = null;
			loader = null;

			Func<Section> sectionFactory;
			if (!AllSections.TryGetValue(name, out sectionFactory))
				throw new ArgumentException("Cannot find section " + name);
			if (!loaders.TryGetValue(name, out loader))
				throw new ArgumentException("Cannot find section " + name);

			section = sectionFactory();
		}

		public static List<KeyValuePair<Section, ILoader>> GetSectionsWithLoaders()
		{
			return homeSections.Keys
				.Select(x => new KeyValuePair<Section, ILoader>(homeSections[x](), loaders[x]))
				.Union(
				sections.Keys
				.Select(x => new KeyValuePair<Section, ILoader>(sections[x](), loaders[x])))
				.ToList();
		}

		public static SectionNames[] GetHomePageSections()
		{
			return homeSections.Keys.ToArray();
		}

		public static Section[] GetTopSections()
		{
			return sections.Values.Select(v => v()).ToArray();
		}
	}

	public enum SectionNames
    {
        Ultimenotiziedamsdn,
        Videosuwindows,
        Informazioni,
    }

	public static class AppInfo {
	public static Uri WebSite { get { return new Uri("http://www.icubed.it/", UriKind.Absolute); } }
	public static bool ShowAd { get { return false; } }
	public static string AdApplicationId { get { return ""; } }
	public static string AdUnit { get { return ""; } }

}
}