using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Common;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using ASPItalia.Apps4Windows.WinRT.Services;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Notifications;

namespace ASPItalia.Apps4Windows.WinRT.Pages
{
	public sealed partial class MainPage : ASPItalia.Apps4Windows.WinRT.Common.LayoutAwarePage
	{
		// ENG: maximum number of elements shownw in home page
		// ITA: numero di elementi visualizzati in home page
		const int MAX_ELEMENTS = 6;

		public MainPage()
		{
			this.InitializeComponent();
		}

		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			base.OnNavigatedTo(e);

			this.Refresh();
		}

		public async override void Refresh()
		{
			// Nascondo il pannello degli errori (se  gi visibile)
			this.ShowError = false;

			try
			{
				try
				{
					// ENG: show the progress ring
					// ITA: Mostro il progress di caricamento
					this.ShowProgress = true;

					// ENG: clear the sections
					// ITA: Azzero le sezioni
					this.AllSections.Clear();

					// ENG: get all sections
					// ITA: Carico la lista dei link di tutte le sezioni
					foreach (Section s in Repository.GetTopSections())
						AllSections.Add(s);

					// ENG: clear the sections
					// ITA: Azzero le sezioni
					this.Sections.Clear();

					// ENG: load all sections
					// ITA: Definisco tutte le sezioni
					foreach (SectionNames s in Repository.GetHomePageSections())
					{
						Section section;
						ILoader loader;

						// ENG: loads section and its items
						// ITA: Carico la sezione con i relativi loader
						Repository.GetSectionAndLoader(s, out section, out loader);

						// ENG: check for ad
						// ITA: verifichiamo se va caricato l'ad
						bool showAd = section.Layout != LayoutTypeEnum.Horizontal && AppInfo.ShowAd;

						try
						{
							// ENG: get all section's items
							// ITA: Carico gli elementi della sezione
							var items = await loader.LoadItemsAsync();

							// ENG: get only the first n items
							// ITA: Aggiungo alla sezione solo i primi n elementi
							SectionsHelper.AddItems(section, items.Take(!showAd ? MAX_ELEMENTS : MAX_ELEMENTS - 1));

							// ENG: se the advertising, if enabled
							// ITA: applichiamo l'advertising, se presente
							if (showAd)
							{
								section.Items.Add(new AdItem(section, MAX_ELEMENTS));
							}

							// ENG: based on the template, we're going to set the number of rows and columns required
							// ITA: Gestiamo i diversi template, indicando quante righe e colonne occupare
							for (int i = 0; i < section.Items.Count; i++)
							{
								SectionItem item = section.Items[i];

								// default
								item.RowSpan = 2;
								item.ColumnSpan = 2;
								item.FillHorizontal = true;
								item.FillVertical = true;

								switch (section.Layout)
								{
									case LayoutTypeEnum.Classic:
										if (i == 0)
										{
											item.RowSpan = 4;
											item.ColumnSpan = 4;
										}
										break;
									case LayoutTypeEnum.Horizontal:
										item.FillVertical = false;
										item.RowSpan = 1;
										item.ColumnSpan = 3;
										break;
								}
							}

							// ENG: let's add the section
							// Aggiungo la sezione alla lista delle sezioni
							this.Sections.Add(section);
						}
						catch (Exception ex)
						{
							// ENG: error, we're going to show a fictious section
							// ITA: errore, mostriamo una sezione finta
							section.HasError = true;
							section.Description = GetErrorMessage(ex);
							this.Sections.Add(section);
						}
					}
				}
				finally
				{
					// ENG: the progress is collapsed anyway
					// ITA: Qualunque cosa succeda nascondo il progress di caricamento
					this.ShowProgress = false;
				}

				// ENG: updates the tile
				// ITA: aggiorno la tile
				UpdateTile();
			}
			catch (Exception ex)
			{
				//this.HandleError(ex);
			}
		}

		ObservableCollection<Section> _sections = new ObservableCollection<Section>();

		public ObservableCollection<Section> Sections
		{
			get
			{
				return _sections;
			}
		}

		ObservableCollection<Section> _allSections = new ObservableCollection<Section>();

		public ObservableCollection<Section> AllSections
		{
			get
			{
				return _allSections;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="navigationParameter"></param>
		/// <param name="pageState"></param>
		protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
		{

		}

		void Header_Click(object sender, RoutedEventArgs e)
		{
			OpenSectionByContext(sender);
		}

		private void OpenSectionByContext(object sender)
		{
			Section section = (Section)(sender as FrameworkElement).DataContext;
			OpenSection(section);
		}

		private void OpenSection(Section section)
		{
			this.Frame.NavigateToSection(section);
		}

		void ItemView_ItemClick(object sender, ItemClickEventArgs e)
		{
			var item = ((SectionItem)e.ClickedItem);
			string parameter = Utils.SerializeValues(item.Section.Name, item.InnerItem.Id);
			if (item.Section.Type == SectionTypeEnum.List)
			{
				this.Frame.Navigate(typeof(ListPage), parameter);
			}
			else
			{
				// ENG: navigate to the details
				// ITA: navigazione al dettaglio
				this.Frame.Navigate(typeof(ItemDetailPage), parameter);
			}
		}

		void Link_Click(object sender, RoutedEventArgs e)
		{
			OpenSectionByContext(sender);
		}


		private void UpdateTile()
		{
			// ENG: this sets the tile cycling
			// ITA: rendiamo possibile il cycling delle tile
			TileUpdateManager.CreateTileUpdaterForApplication().EnableNotificationQueue(true);

			var allItems = Sections.SelectMany(x => x.Items).OrderByDescending(x => x.PubDate).Take(5);
			foreach (var item in allItems)
			{
				TileHelper.UpdatePrimaryTile(item, new System.Uri("ms-appx:///Assets/WideLogo.png"), new System.Uri("ms-appx:///Assets/Logo.png"));
			}
		}

		private void horizontalGrid_ViewChangeStarted(object sender, SemanticZoomViewChangedEventArgs e)
		{
			Visibility v;
			if (e.IsSourceZoomedInView)
			{
				v = Windows.UI.Xaml.Visibility.Collapsed;
			}
			else
			{
				v = Windows.UI.Xaml.Visibility.Visible;
			}

			// ITA: hide or show the AdButton
			// ENG: Nascondo o mostro tutti gli AdButton (perch non scompaiono)
			foreach (AdButton b in horizontalGrid.GetDescendants().OfType<AdButton>())
			{
				b.Visibility = v;
			}
		}
	}
}
